/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

import java.io.*;
import java.util.*;

import palmbible.parser.*;
import palmbible.pdbfile.*;
import palmbible.compress.*;
import palmbible.util.*;
import palmbible.data.*;

import palmbible.output.*;

class BibleConverter
{
	public BibleConverter(CommandLine commandLine)
	{
		try {
			String fileName = commandLine.getInputFile();
	
			File file = new File(fileName + ".txt");
			
			String encode = commandLine.getEncoding();
			String decode = commandLine.getDecoding();
			
			boolean quite = commandLine.getQuite();

			System.out.println();
			System.out.println("Parsing Bible Source ...");
			System.out.println();

			BibleDoc bibleDoc = new BibleDoc(file, decode, encode, quite);
			
			System.out.println();
			System.out.println("Total Book is ... " + bibleDoc.getTotalBook());
			System.out.println();

			boolean exported = false;

			if (commandLine.exportPC()){
				exported = true;
			}

			if (commandLine.exportPlain()){
				new PlainDoc(bibleDoc, fileName);
				exported = true;
			}

			if (commandLine.exportHtml()){
				new HTMLDoc(bibleDoc, fileName);
				exported = true;
			}

			if (commandLine.exportRtf()){
				new RTFDoc(bibleDoc, fileName);
				exported = true;
			}

			if (commandLine.exportThml()){
				new THMLDoc(bibleDoc, fileName);
				exported = true;
			}

			if (commandLine.exportMoNoon()){
				new MoNoonDoc(bibleDoc, fileName);
				exported = true;
			}
			
			if (commandLine.exportPdb() || !exported){
				new PDBDoc(bibleDoc, fileName);
				exported = true;
			}

		} catch (ParserException e) {
			System.err.println("Parser Exception : " + e.getMessage());
			e.printStackTrace();
		} catch (NullPointerException e) {
			System.err.println("Unexpected Syntax Exception : " + e.getMessage());
			e.printStackTrace();
		} catch (Exception e) {
			System.err.println("Unexpected Exception : " + e.getMessage());
			e.printStackTrace();
		}
	}

	public static void main(String args[])
	{
		System.out.println();
		System.out.println(Info.version);
		System.out.println(Info.copyright);

		CommandLine commandLine = null;
		try {
			commandLine = new CommandLine(args);
		} catch (Exception e) {}

		if (commandLine == null) {
			System.out.println();
			System.out.println("FREEWARE");
			System.out.println("Please read copyright.txt and license.txt before use.");
			System.out.println();
			System.out.println("PDB File Converter For Bible Reader");
			System.out.println("Please visit http://palmbible.sourceforge.net/");
			System.out.println();
			System.out.println("Usage :");
			System.out.println();
			System.out.println("java BibleConverter <param> <input>");
			System.out.println();
			System.out.println("<param>");
			System.out.println("-q                 quite mode");
			System.out.println("-pdb               generate pdb file for Bible Reader (default)");
			System.out.println("-plain             generate plain text file");
			System.out.println("-html              generate html files into under <input> directory");
			System.out.println("-rtf               generate rtf files into under <input> directory");
			System.out.println();
			System.out.println("-encode <encoding> overide document output encoding");
			System.out.println("-decode <encoding> overide document input encoding");
			System.out.println();
			System.out.println("<input> is the name of <input>.txt");
			System.out.println();
			System.out.println("Remark :");
			System.out.println("<input>.txt must be located in the Bible Converter directory");
			System.out.println();
		} else {
			BibleConverter bibleConverter = new BibleConverter(commandLine);
		}
	}
}
