package palmbible.output;

import java.io.*;
import java.util.*;

import palmbible.parser.*;
import palmbible.util.*;

public class THMLDoc extends GenericDoc
{
	protected Template headTP;
	protected Template footTP;
	protected Template testamentHeadTP;
	protected Template testamentFootTP;
	protected Template bookHeadTP;
	protected Template bookFootTP;
	protected Template chapterHeadTP;
	protected Template chapterFootTP;
	protected Template verseHeadTP;
	protected Template verseFootTP;
	protected Template titleTP;
	protected Template verseTP;

	boolean accept = true;
	boolean testOldHead = false;
	boolean testNewHead = false;

	public THMLDoc(BibleDoc bibleDoc, String docName) throws Exception
	{
		super(bibleDoc, docName);
	
		setTemplatePackage("/template/thml/");

		headTP = getTemplate("head");
		footTP = getTemplate("foot");
		testamentHeadTP = getTemplate("testament.head");
		testamentFootTP = getTemplate("testament.foot");
		bookHeadTP = getTemplate("book.head");
		bookFootTP = getTemplate("book.foot");
		chapterHeadTP = getTemplate("chapter.head");
		chapterFootTP = getTemplate("chapter.foot");
		verseHeadTP = getTemplate("verse.head");
		verseFootTP = getTemplate("verse.foot");
		titleTP = getTemplate("title");
		verseTP = getTemplate("verse");

		encode = "UTF8";
		generate();
	}

	protected void beginBible(Map wordMap) throws Exception
	{
		System.out.println("Making THML Files ...");

		setOutputDir(".");
		setOutputFile(wordMap.get("%%docname%%") + ".thml");

		print(headTP.getString(wordMap));
	}

	protected void endBible(Map wordMap) throws Exception
	{
		if (testNewHead){
			print(testamentFootTP.getString(wordMap));
		} else if (testOldHead){
			print(testamentFootTP.getString(wordMap));
		}

		print(footTP.getString(wordMap));

		System.out.println("THML Files Generated ...");
		System.out.println();
	}

	protected void beginBook(Map wordMap) throws Exception
	{
		int bookSeq = Integer.parseInt((String) wordMap.get("%%bookseq%%"));

		if (bookSeq <= 39){
			if (!testOldHead){
				wordMap.put("%%testamenttext%%", "Old Testament");
				wordMap.put("%%testamentnum%%", "1");
				print(testamentHeadTP.getString(wordMap));
				testOldHead = true;
			}
		} else {
			if (!testNewHead){
				wordMap.put("%%testamentnum%%", "1");
				if (testOldHead){
					print(testamentFootTP.getString(wordMap));
					wordMap.put("%%testamentnum%%", "2");
				}
				wordMap.put("%%testamenttext%%", "New Testament");
				print(testamentHeadTP.getString(wordMap));
				testNewHead = true;
			}
		}

		if (bookSeq > 66) {
			clearStored();
			accept = false;
		} else {
			accept = true;
		}

		if (accept){
			print(bookHeadTP.getString(wordMap));
		}
	}
	
	protected void endBook(Map wordMap) throws Exception
	{
		if (accept){
			print(bookFootTP.getString(wordMap));
		}
		accept = true;
	}
	
	protected void beginChapter(Map wordMap) throws Exception
	{
		if (accept){
			print(chapterHeadTP.getString(wordMap));
		}
	}

	protected void endChapter(Map wordMap) throws Exception
	{
		if (accept){
			print(chapterFootTP.getString(wordMap));
		}
	}

	protected void beginVerse(Map wordMap) throws Exception
	{
		if (accept){
			print(verseHeadTP.getString(wordMap));
		}
	}
	
	protected void endVerse(Map wordMap) throws Exception
	{
		if (accept){
			print(verseFootTP.getString(wordMap));
		}
	}
	
	protected void verseText(Map wordMap) throws Exception
	{
		if (accept){
			print(verseTP.getString(wordMap));
		}
	}
	
	protected void titleText(Map wordMap) throws Exception
	{
		if (accept){
			print(titleTP.getString(wordMap));
		}
	}
}
