/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

import java.io.*;

public class BibleDoc extends Parser
{
	protected static final String TAG_NAME = "BIBLE";
	protected WordRecorder wordRecorder;
	protected boolean breakUnicode = false;
	protected String encode = "iso-8859-1";
	protected String decode = "iso-8859-1";
	
	protected boolean quite = false;
	
	protected BibleChecker bibleCheck = new BibleChecker();

	public BibleDoc(File file, String dec, String enc, boolean quite) throws Exception
	{
		this.quite = quite;
		
		InputStream inputStream = new FileInputStream(file);
		Tokenizer tokenizer = new Tokenizer(inputStream, encode);
		ParserInfo parserInfo = new ParserInfo(tokenizer);
		tokenizer.close();

		if (dec == null){
			decode = parserInfo.getDecoding();
		} else {
			decode = dec;
		}
		
		if (enc == null){
			encode = parserInfo.getEncoding();
		} else {
			encode = enc;
		}

		inputStream = new FileInputStream(file);
		tokenizer = new Tokenizer(inputStream, decode);

		wordRecorder = new WordRecorder(encode);
		parserInfo = new ParserInfo(tokenizer);
		
		breakUnicode = parserInfo.getBreakUnicode();
		tokenizer.setBreakUnicode(breakUnicode);
		
		checkTag(tokenizer);
		parse(tokenizer, wordRecorder);
		tokenizer.close();
		
		sortBook();
	}
	
	public String getEncoding()
	{
		return encode;
	}
	
	public WordRecorder getWordRecorder()
	{
		return wordRecorder;
	}
	
	public boolean getBreakUnicode()
	{
		return breakUnicode;
	}

	public boolean getRightAligned()
	{
		String align = getStringAttrib("ALIGN");

		if (align != null && align.toUpperCase().equals("RIGHT")){
			return true;
		} else {
			return false;
		}
	}

	public boolean getByteShifted()
	{
		return wordRecorder.getByteShifted();
	}

	public boolean getCopyProtected()
	{
		return false;
	}

	public String getBibleDocName()
	{
		return getStringAttrib("NAME");
	}

	public String getBibleDocInfo()
	{
		String info = getStringAttrib("INFO");
		
		if (info == null){
			return "";
		} else {
			return info;
		}
	}
	
	public int getTotalBook()
	{
		return getTotalChild();
	}

	public Book getBook(int index)
	{
		return (Book) getChild(index);
	}

	public String getTagName()
	{
		return TAG_NAME;
	}

	protected void sortBook()
	{
		int totalBook = getTotalBook();
		Book[] books = new Book[totalBook];
		
		for (int i=0; i < totalBook; i++){
			books[i] = getBook(i);
		}
		
		for (int i=0; i < totalBook; i++){
			for (int j=0; j < totalBook - 1; j++){
				if (books[j].getBookNumber() > books[j + 1].getBookNumber()){
					Book book = books[j];
					books[j] = books[j+1];
					books[j+1] = book;
				}
			}
		}
		
		childs.removeAllElements();

		for (int i=0; i < totalBook; i++){
			childs.addElement(books[i]);
		}
	}
	
	protected Object parseChild(Tokenizer tokenizer, WordRecorder wordRecorder) throws Exception
	{
		Book book = new Book(tokenizer, wordRecorder);

		if (!quite){
			bibleCheck.checkBook(book);
		}
		return book;
	}
}
