/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

public class ParserInfo
{
	protected static final String ENCODE_DEFAULT = "iso-8859-1";

	protected static final String CONT_TYPE = "CONT";
	protected static final String SPCSEP_TYPE = "SPCSEP";
	protected static final String PARSERINFO_TAG = "PARSERINFO";
	protected Tag tag;

	public ParserInfo(Tokenizer tokenizer) throws Exception
	{
		Token token = tokenizer.getToken();
		while (!token.isTag()){
			token = tokenizer.getToken();
		}
		
		if (token.isTag()){
			tag = token.getTag();
			
			if (tag.getName().equals(PARSERINFO_TAG)){
				return;
			}
		}

		throwNoParserInfo();
	}
	
	public String getEncoding()
	{
		String encode = tag.getAttrib("ENCODE");
		
		if (encode == null){
			encode = ENCODE_DEFAULT;
		}
		
		return encode;
	}

	public String getDecoding()
	{
		String decode = tag.getAttrib("DECODE");
		
		if (decode == null){
			decode = getEncoding();
		}

		return decode;
	}

	public boolean getBreakUnicode() throws ParserException
	{
		String wordType = tag.getAttrib("WORDTYPE");
		
		if (wordType.equals(CONT_TYPE)){
			return true;
		} else if (wordType.equals(SPCSEP_TYPE)){
			return false;
		} else {
			throwUnknownWordType(wordType);
		}
		
		return true;
	}

	protected void throwUnknownWordType(String wordType) throws ParserException
	{
		throw new ParserException("Unknow Word Type : " + wordType);
	}

	protected void throwNoParserInfo() throws ParserException
	{
		throw new ParserException("Unable to find parser info.");
	}
}
