/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

import java.util.*;

public class Tag
{
	public static final String OPEN_TAG = "<";
	public static final String SLASH_TAG = "/";
	public static final String CLOSE_TAG = ">";
	public static final String EQUALS_TAG = "=";
	public static final String QUOTES_TAG = "\"";

	protected boolean isOpenTag;
	protected String name;
	protected Map attrib = new HashMap();

	public Tag(Tokenizer tokenizer) throws Exception
	{
		boolean spaceToken = tokenizer.getSpaceToken();
		tokenizer.setSpaceToken(false);

		Token token = tokenizer.getToken();
		name = token.toString();
		
		if (name.equals(SLASH_TAG)){
			token = tokenizer.getToken();
			name = token.toString();
			isOpenTag = false;
		} else {
			isOpenTag = true;
		}

		String attribName = null;
		String attribValue = null;

		token = tokenizer.getToken();
		attribName = token.toString();

		while (!attribName.equals(CLOSE_TAG)){
			token = tokenizer.getToken();
			attribValue = token.toString();

			if (attribValue.equals(EQUALS_TAG)){
				token = tokenizer.getToken();
				attribValue = token.toString();
				
				if (attribValue.equals(QUOTES_TAG)){
					StringBuffer buffer = new StringBuffer();

					tokenizer.setSpaceToken(true);

					token = tokenizer.getToken();
					attribValue = token.toString();

					while (!attribValue.equals(QUOTES_TAG)){
						buffer.append(attribValue);
						
						token = tokenizer.getToken();
						attribValue = token.toString();
					}
					attribValue = buffer.toString();

					tokenizer.setSpaceToken(false);
				}
			}
			attrib.put(attribName.toUpperCase(), attribValue);

			token = tokenizer.getToken();
			attribName = token.toString();
		}

		tokenizer.setSpaceToken(spaceToken);
	}
	
	public String getName()
	{
		return name;
	}
	
	public boolean isOpenTag()
	{
		return isOpenTag;
	}

	public boolean isCloseTag()
	{
		return !isOpenTag;
	}
	
	public String getAttrib(String name)
	{
		return (String) attrib.get(name.toUpperCase());
	}
	
	public String toString()
	{
		StringBuffer buffer = new StringBuffer();
		
		buffer.append(OPEN_TAG);
		if (isCloseTag()){
			buffer.append(SLASH_TAG);
		}
		buffer.append(name);
		
		Set keySet = attrib.keySet();
		Iterator keys = keySet.iterator();
		
		while (keys.hasNext()){
			buffer.append(' ');
			String key = (String) keys.next();
			buffer.append(key);
			buffer.append("=\"");
			String value = (String) attrib.get(key);
			buffer.append(value);
			buffer.append('\"');
		}
		
		buffer.append(CLOSE_TAG);
		
		return buffer.toString();
	}
}
