/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

import java.util.*;
import palmbible.util.*;

public class WordRecorder
{
	public static final int BYTE_SHIFTED_LIMIT = 0x36FF;
	public static final int BYTE_SHIFTED_MAX = 0x3FE0;
	public static final int BYTE_UNSHIFTED_MAX = 0xFFE0;

	protected String enc;
	protected int wordCount;
	protected Map words = new TreeMap();
	protected Vector indexs = new Vector();

	protected Map sortIndex = new HashMap();
	protected Vector sorted = new Vector();
	
	protected TreeNode treeRoot;

	public WordRecorder(String enc)
	{
		this.enc = enc;
	}
	
	public int getWordIndex(String word) throws Exception
	{
		Integer indexObj = (Integer) words.get(word);

		if (indexObj == null){
			wordCount++;
			indexObj = new Integer(wordCount);
			words.put(word, indexObj);
			indexs.addElement(word);
			
			TreeNode treeNode = new TreeNode(word, wordCount, enc);
			if (treeRoot == null){
				treeRoot = treeNode;
			} else {
				treeNode.putNode(treeRoot);
			}
			
			if (wordCount > BYTE_UNSHIFTED_MAX){
				throwWordCountLimitExceeded();
			}
			
			return wordCount;
		} else {
			return indexObj.intValue();
		}
	}
	
	public String getWordString(int index)
	{
		return (String) indexs.elementAt(index - 1);
	}
	
	public int getWordCount()
	{
		return wordCount;
	}
	
	public int getNewWordIndex(int index)
	{
		if (index <= 0xFFF0){
			Integer indexObj = (Integer) sortIndex.get(new Integer(index));
			return indexObj.intValue();
		} else {
			return index;
		}
	}
	
	public String getNewWordString(int index)
	{
		if (index < 0xFFF0){
			return (String) sorted.elementAt(index - 1);
		} else {
			return "";
		}
	}
	
	public void sort()
	{
		treeRoot.fillSorted(sorted, sortIndex);
	}
	
	public boolean getByteShifted()
	{
		if (wordCount < WordRecorder.BYTE_SHIFTED_LIMIT){
			return true;
		} else {
			return false;
		}
	}

	protected void throwWordCountLimitExceeded() throws ParserException
	{
		throw new ParserException("Word Count Limit Exceeded. You may try breaking the Bible version into parts.");
	}
}
