/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.pdbfile;

import java.util.*;
import palmbible.parser.*;
import palmbible.compress.*;
import palmbible.util.*;

public class WordRecords
{
	protected static final int recordSize = 4096;
	protected Vector content = new Vector();
	protected Vector wordIndex = new Vector();

	public WordRecords(WordRecorder wordRecorder, Compressor compressor)
	{
		WordRecord wordRecord = new WordRecord();
		content.addElement(wordRecord);

		int totalWord = wordRecorder.getWordCount();
		int charIndex = 0;

		for(int i=0; i<totalWord; i++){
			String word = wordRecorder.getNewWordString(i + 1);
			int totalChar = word.length();
			
			if (charIndex + totalChar <= recordSize){
				wordRecord.addByte(word);
				charIndex += totalChar;
			} else {
				int trails = recordSize - charIndex;
				
				String part1 = word.substring(0, trails);
				String part2 = word.substring(trails);

				charIndex = totalChar - trails;

				wordRecord.addByte(part1);

				wordRecord = new WordRecord();
				content.addElement(wordRecord);

				wordRecord.addByte(part2);
			}
		}

		int totalCompressedWord = compressor.getTotalCompressedWord();

		for(int i=0; i<totalCompressedWord; i++){
			String word = compressor.getCompressedWord(i + 1);
			int totalChar = word.length();

			for (int j=0; j< totalChar; j++){
				if (charIndex >= recordSize){
					wordRecord = new WordRecord();
					content.addElement(wordRecord);
					charIndex = 0;
				}
				
				int compressedChar = word.charAt(j);
				int upperByte = compressedChar >> 8;
				wordRecord.addByte(upperByte);
				charIndex ++;

				if (charIndex >= recordSize){
					wordRecord = new WordRecord();
					content.addElement(wordRecord);
					charIndex = 0;
				}

				int lowerByte = compressedChar;
				wordRecord.addByte(lowerByte);
				charIndex ++;
			}
		}
	}
	
	public int getTotalRecord()
	{
		return content.size();
	}

	public WordRecord getRecord(int index)
	{
		return (WordRecord) content.elementAt(index);
	}
}
