/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"

static WordInfoPtr decWordInfo;

static FullReaderPtr decWordReader;
static FullReaderPtr decVerseReader;

static UInt32 decVerseStart;
static UInt16 decShift;
static UInt32 decValueBuffer[3];

static UInt16 decScreenPos, decLinePos, decCharPos;
static Boolean decTyped, decBlankAfter;

static char* decCharBuffer;
static char* decLineBuffer;
static char* decVerseBuffer[128];

static UInt16 decCompressedBuffer[16];

static UInt32 decWordIndex; 
static UInt16 decWordNum, decWordLen;
static UInt16 decRepeat, decCompressedIndex;
static UInt16 decByteShifted;

static Boolean decCompressed;
static Boolean decVBoldEnable;

static char decSepChar;
static char decLastType;

static UInt16 getNumWidth(UInt16 value)
{
	UInt16 averageFontWidth;
	UInt16 numWidth;
	UInt16 spcCharWidth;
	char* numBuffer;
	
	RenSetTextFont(decVBoldEnable);
	averageFontWidth = FntAverageCharWidth();

	numBuffer = MemPtrNew(8);
	StrIToA(numBuffer, value);
	StrCat(numBuffer, " ");
	
	RenSetVerseFont();
	numWidth = FntCharsWidth(numBuffer, StrLen(numBuffer));
	RenSetTextFont(decVBoldEnable);

	MemPtrFree(numBuffer);
	
	spcCharWidth = FntCharWidth(' ');

	if (vBold){
		averageFontWidth += spcCharWidth;
	}
	
	if (wordSpace > 0){
		averageFontWidth += spcCharWidth;
	}
	
	if (wordSpace > 1){
		averageFontWidth += spcCharWidth;
	}
	
	numWidth = (numWidth / averageFontWidth + 1) * averageFontWidth;
	
	return numWidth;
} 

static void getWordIndexLen()
{
	UInt32 totalByteAcc;
	UInt16 totalIndexes, relNum, totalWord, wordLen, i;
	Boolean compressed;

	totalIndexes = decWordInfo->totalIndexes;
	
	decWordIndex = 0;
	decWordLen = 0;

	totalByteAcc = 0;
	totalWord = 0;
	
	compressed = false;
	
	relNum = decWordNum - 1;
	for (i=0; i<totalIndexes; i++){
		totalWord = decWordInfo->wordIndex[i].totalWord;
		wordLen = decWordInfo->wordIndex[i].wordLength;
		compressed = decWordInfo->wordIndex[i].compressed;

		if (relNum < totalWord){
			decWordLen = wordLen;
			decWordIndex = totalByteAcc + relNum * wordLen;
			i = totalIndexes;
		} else {
			totalByteAcc += totalWord * wordLen;
			relNum -= totalWord;
		}
	}

	decCompressed = compressed;
}

static void beginVerse()
{
	UInt32 compStart;
	UInt16 shiftLookup[] = {0, 3, 2, 1};

	if (decByteShifted){
		compStart = decVerseStart * 7 / 4;
		decShift = shiftLookup[decVerseStart * 7 % 4];
	} else {
		compStart = decVerseStart * 2;
		decShift = 0;
	}

	FrdReadBegin(decVerseReader, compStart);

	if (decByteShifted){
		switch(decShift){
			case 1:
				decValueBuffer[1] = FrdReadChar(decVerseReader);
				break;
			case 2:
				decValueBuffer[2] = FrdReadChar(decVerseReader);
				break;
			case 3:
				decValueBuffer[2] = FrdReadChar(decVerseReader);
				break;
			default:
		}
	}
}

static UInt16 readVerse()
{
	UInt16 shiftLookup[] = {10, 4, 6, 8};
	UInt32 value;
	
	if (decByteShifted){
	switch(decShift){
		case 0:
			decValueBuffer[0] = FrdReadChar(decVerseReader);
			decValueBuffer[1] = FrdReadChar(decVerseReader);
			decValueBuffer[2] = 0;
			break;
		case 1:
			decValueBuffer[0] = decValueBuffer[1];
			decValueBuffer[1] = FrdReadChar(decVerseReader);
			decValueBuffer[2] = FrdReadChar(decVerseReader);
			break;
		case 2:
			decValueBuffer[0] = decValueBuffer[2];
			decValueBuffer[1] = FrdReadChar(decVerseReader);
			decValueBuffer[2] = FrdReadChar(decVerseReader);
			break;
		case 3:
			decValueBuffer[0] = decValueBuffer[2];
			decValueBuffer[1] = FrdReadChar(decVerseReader);
			decValueBuffer[2] = 0;
			break;
		default:
	}

	value = decValueBuffer[0] << 16 | decValueBuffer[1] << 8 | decValueBuffer[2];
	value = value >> shiftLookup[decShift];
	value = value & 0x3FFF;
	decShift ++;

	if (decShift == 4){
		decShift = 0;
	}
	
	if (value > 0x3FF0){
		value |= 0xC000;
	}

	decWordNum = value;
	} else {
		decWordNum = FrdReadChar(decVerseReader) * 256 + FrdReadChar(decVerseReader);
	}
}

static void endVerse()
{
	FrdReadEnd(decVerseReader);
}

static void beginWord()
{
	decCharBuffer = MemPtrNew(64);
}

static void readWord()
{
	UInt16 i;
	char charValue;

	FrdReadBegin(decWordReader, decWordIndex);
	for (i=0; i<decWordLen; i++){
		charValue = FrdReadChar(decWordReader);
		decCharBuffer[i] = charValue;
	}
	decCharBuffer[i] = 0;
	FrdReadEnd(decWordReader);
}

static void endWord()
{
	 MemPtrFree(decCharBuffer);
}

static void checkCompressedWordNum()
{
	UInt16 numValue, i;

	if (decWordNum < 0xFFF0){
		getWordIndexLen();

		if (decCompressed){
			decRepeat = decWordLen / 2;
			FrdReadBegin(decWordReader, decWordIndex);
			for (i=0; i<decRepeat; i++){
				numValue = FrdReadChar(decWordReader) * 256 + FrdReadChar(decWordReader);
				decCompressedBuffer[i] = numValue;
			}
			FrdReadEnd(decWordReader);
		} else {
			decRepeat = 1;
			decCompressedBuffer[0] = decWordNum;
		}
	} else {
		decRepeat = 1;
		decCompressedBuffer[0] = decWordNum;
	}
	
	decCompressedIndex = 0;
}

static void readCompressed()
{
	decWordNum = decCompressedBuffer[decCompressedIndex];
	if (decByteShifted && decWordNum > 0x3FF0){
		decWordNum |= 0xC000;
	}
	decCompressedIndex ++;
}

void addBlankLine();

static void setTyped()
{
	if (decTyped && decCharPos > 0){
		if (decBlankAfter && decLastType != decLineBuffer[0] && 
			((dualView && !noBlankDual) || (!dualView && !noBlankNormal))){
			decBlankAfter = false;
			decVerseBuffer[decLinePos] = MemPtrNew(2);
			decVerseBuffer[decLinePos][0] = 'n';
			decVerseBuffer[decLinePos][1] = 0;
			decLinePos++;
			decTyped = true;
		}
		
		decLastType = decLineBuffer[0];

		decLineBuffer[decCharPos] = 0;
		decVerseBuffer[decLinePos] = MemPtrNew(decCharPos + 1);
		StrCopy(decVerseBuffer[decLinePos], decLineBuffer);
		decLinePos++;
		
		decCharPos = 0;
		decScreenPos = 0;
	}
}

void addBlankLine()
{
	if (!decTyped){
		if ((dualView && !noBlankDual) || (!dualView && !noBlankNormal)){
			if (decCharPos > 1){
				setTyped();
			}

			decLineBuffer[0] = 'n';
			decLineBuffer[1] = 0;
			decCharPos = 1;
		
			decTyped = true;
			setTyped();
			decTyped = false;
		}
	}
}

void addSpace()
{
    if (decSepChar != 0){
    	decCharBuffer[decWordLen] = decSepChar;
    	decWordLen++;
    } else if (vBold){
    	decCharBuffer[decWordLen] = ' ';
    	decWordLen++;
    }
    
    if (wordSpace > 0){
    	decCharBuffer[decWordLen] = ' ';
    	decWordLen++;
    }
    
    if (wordSpace > 1){
    	decCharBuffer[decWordLen] = ' ';
    	decWordLen++;
    }
}

void DecOpenDecoder(DecoderPtr decoder, UInt16 version)
{
	BookInfoPtr bookInfo;

	decoder->recordIndex = InfGetWordRecordIndex(version);
	decoder->totalRecord = InfGetWordTotalRecord(version);
	decoder->sepChar = InfGetSepChar(version);
	decoder->byteShifted = InfGetByteShifted(version);
	decoder->version = version;
	
	FrdOpenReader(&decoder->wordReader, decoder->version, decoder->recordIndex + 1, 
		decoder->recordIndex + decoder->totalRecord);
		
	decoder->wordInfo = DmxLockVersionRecord(decoder->version, decoder->recordIndex);
}

void DecCloseDecoder(DecoderPtr decoder)
{
	DmxUnlockVersionRecord(decoder->version, decoder->recordIndex);
	FrdCloseReader(&decoder->wordReader);
}

void DecDecodeVerseAsString(DecoderPtr decoder, char* verse, 
	FullReaderPtr recordReader, UInt32 verseStart, UInt16 verseLength)
{
	UInt16 charsWidth, i, j;
	char* verseWord;
	char typeChar;
	Boolean addWord;
	
	addWord = true;
	
	decSepChar = decoder->sepChar;
	decByteShifted = decoder->byteShifted;

	decWordInfo = decoder->wordInfo;
	decWordReader = &decoder->wordReader;
	decVerseReader = recordReader;
	
	decVerseStart = verseStart;

	decLinePos = 0;
	decCharPos = 0;
	
	beginWord();
	beginVerse();
	
	decLineBuffer = verse;

	for (i=0; i<verseLength; i++) {
		readVerse();
		checkCompressedWordNum();
		
		for (j=0; j<decRepeat; j++){
    		readCompressed();
    		switch (decWordNum){
    			case bookTextType :
    				addWord = false;
    				break;
    			case chapTextType :
    				addWord = false;
    				break;
    			case descTextType :
    				addWord = false;
    				break;
    			case versTextType :
    				addWord = true;
    				break;
    			default:
    				if (addWord){
	   					getWordIndexLen();

    					if (decCharPos + decWordLen < searchBufferSize - 1){
    						readWord();
    					
    						if (decSepChar != 0){
    							decCharBuffer[decWordLen] = decSepChar;
		   						decWordLen++;
    							decCharBuffer[decWordLen] = 0;
    						}
    
    						StrCopy(&verse[decCharPos], decCharBuffer);
    						decCharPos += decWordLen;
    					}
    				}
    		}
		}
	}

	verse[decCharPos] = 0;

	endVerse();
	endWord();
}

void DecDecodeVerse(DecoderPtr decoder, VersePtr verse, 
	FullReaderPtr recordReader, UInt32 verseStart, UInt16 verseLength)
{
    UInt16 blankSpaceWidth;
	UInt16 charsWidth, i, j;
	char* verseWord;
	char typeChar;

	decSepChar = decoder->sepChar;
	decByteShifted = decoder->byteShifted;

	decVBoldEnable = (decSepChar == 0);
	
	decWordInfo = decoder->wordInfo;
	decWordReader = &decoder->wordReader;
	decVerseReader = recordReader;
	
	decVerseStart = verseStart;

	decScreenPos = 0;
	decCharPos = 0;
	decLinePos = 0;
	decTyped = false;
	decBlankAfter = false;
	
	decLastType = 0;

	beginWord();
	beginVerse();
	
	RenSetTextFont(decVBoldEnable);

	decLineBuffer = MemPtrNew(128);
	for (i=0; i<verseLength; i++) {
		readVerse();
		checkCompressedWordNum();
		
		for (j=0; j<decRepeat; j++){
    		readCompressed();

    		switch (decWordNum){
    			case bookTextType :
    				setTyped();
    				decLineBuffer[0] = 'b';
    				decLineBuffer[1] = 0;
    				decCharPos = 1;
    				decBlankAfter = true;
					decLastType = 'b';
    				decTyped = true;
    				RenSetBookFont(decVBoldEnable);
    				break;
    			case chapTextType :
    				setTyped();
    				addBlankLine();
    
    				decLineBuffer[0] = 'c';
    				decLineBuffer[1] = 0;
    				decCharPos = 1;
    				decTyped = true;
    				RenSetChapFont(decVBoldEnable);
    				break;
    			case descTextType :
    				setTyped();
    				addBlankLine();
    
    				decLineBuffer[decCharPos] = 'd';
    				decLineBuffer[1] = 0;
    				decCharPos = 1;
    				decTyped = true;
    				RenSetDescFont(decVBoldEnable);
    				break;
    			case versTextType :
    				setTyped();
    				decTyped = true;
    				RenSetTextFont(decVBoldEnable);
    				break;
    			default:
    				if (decCharPos == 0) {
    					decLineBuffer[0] = 's';
    					
    					charsWidth = getNumWidth(verse->verse);
    					
    					decLineBuffer[1] = charsWidth;
    					decLineBuffer[2] = 0;
    					decScreenPos = charsWidth;
    					decCharPos = 2;
    					decTyped = true;
    				}
                    blankSpaceWidth = FntCharWidth(' ');

    				getWordIndexLen();
    				readWord();

					if (decWordLen == 1 && decCharPos > 3 && decSepChar != 0){
						switch (decCharBuffer[0]){
							case '.': case ',': 
							case '?': case '!': 
							case ';': case ':':
								decCharPos --;
								decScreenPos -= blankSpaceWidth;  
								break; 
						}
					}
					addSpace();
    
    				charsWidth = FntCharsWidth(decCharBuffer, decWordLen);
    				decScreenPos += charsWidth;
    
    				if (decScreenPos > screenWidth + blankSpaceWidth * wordSpace){
    					typeChar = decLineBuffer[0];
    					if (typeChar == 's'){
	    					setTyped();
    						decLineBuffer[0] = 'n';
    					} else {
	    					setTyped();
    						decLineBuffer[0] = typeChar;
    					}
    					decLineBuffer[1] = 0;
    					decCharPos = 1;
    					decScreenPos = charsWidth;
    				}
    
    				StrCopy(&decLineBuffer[decCharPos], decCharBuffer);
    				decCharPos += decWordLen;
    		}
    	}
	}

	if (decLinePos == 0 && decCharPos <= 1){
		verse->lines = 0;
	} else {
		if (decCharPos > 1){
			decTyped = true;
			setTyped();
		}
		
		verse->textLines = MemPtrNew(sizeof(char*) * decLinePos);
		
		for (i=0; i < decLinePos; i++){
			verse->textLines[i] = decVerseBuffer[i];
		}

		verse->lines = decLinePos;
	}
	MemPtrFree(decLineBuffer);

	endVerse();
	endWord();
}

