/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"

static FormPtr form;

static Coord lastY;
static Boolean upperView;
static Boolean penDowned;
static Boolean redrawTitle;
static WinHandle bufferWin;

static char* fmnTitle;

static UInt16 lastChapter, lastVerse;

static void drawTitle(char* title)
{
	IndexedColorType orgTextColor, orgForeColor, orgBackColor;

	RectangleType fmnTitleBound;
	WinHandle titleWin;
	WinHandle drawWin;
	UInt16 titleWidth;
	UInt16 error;
	
	fmnTitleBound.topLeft.x = 0;
	fmnTitleBound.topLeft.y = 0;
	
	fmnTitleBound.extent.x = maxTitleWidth;
	fmnTitleBound.extent.y = 16;

	titleWin = WinCreateOffscreenWindow(maxTitleWidth, 13, screenFormat, &error);
	drawWin = WinSetDrawWindow(titleWin);
	
	FntSetFont(1);
	titleWidth = FntCharsWidth(title, StrLen(title));
	
	WinEraseRectangle(&fmnTitleBound, 0);

	if (titleWidth < maxTitleWidth - 5){
		fmnTitleBound.extent.x = titleWidth + 5; 
	}

	if (os35){
		orgForeColor = WinSetForeColor(
			UIColorGetTableEntryIndex(
			UIMenuSelectedFill));

		orgBackColor = WinSetBackColor(
			UIColorGetTableEntryIndex(
			UIMenuSelectedFill));
    
		orgTextColor = WinSetTextColor(orgBackColor);
	
		WinDrawRectangle(&fmnTitleBound, 3);
		WinDrawTruncChars(title, StrLen(title), 3, 2, maxTitleWidth - 5);

		WinSetForeColor(orgForeColor);
		WinSetBackColor(orgBackColor);
		WinSetTextColor(orgTextColor);
	} else {
		WinDrawTruncChars(title, StrLen(title), 3, 2, maxTitleWidth - 5);
		WinInvertRectangle(&fmnTitleBound, 3);
	}
	
	fmnTitleBound.extent.x = maxTitleWidth;
	fmnTitleBound.extent.y = 13;

	WinCopyRectangle(titleWin, drawWin, &fmnTitleBound, 0, 0, winPaint);
	WinSetDrawWindow(drawWin);

	WinDeleteWindow(titleWin, false);
	
	FntSetFont(0);
}

static void drawTitleVerse()
{
	char* verseText;
	
	verseText = InfGetSimpleFormattedVerse(upperViewer.version, 
		upperViewer.book, upperViewer.chapter, upperViewer.verse);
		
	drawTitle(verseText);
	
	MemPtrFree(verseText);
}

static void redrawViewer()
{
	WinHandle drawWin;
	
	modalForm = form;

	if (history.totalForward > 0){
		setControlUsable(btnForward, true);
	} else {
		setControlUsable(btnForward, false);
	}

	if (history.totalBack > 0){
		setControlUsable(btnBack, true);
	} else {
		setControlUsable(btnBack, false);
	}

	if (doubleBuffered){
		drawWin = WinSetDrawWindow(bufferWin);
	}

	if (dualView){
		WinEraseRectangle(&(fullBound), 0);
		upperViewer.bound = upperBound;
		VwrRedrawViewer(&upperViewer);

		WinDrawLine(upperViewer.bound.topLeft.x, 
			upperViewer.bound.topLeft.y + upperViewer.bound.extent.y + 2,
			upperViewer.bound.topLeft.x + upperViewer.bound.extent.x, 
			upperViewer.bound.topLeft.y + upperViewer.bound.extent.y + 2);

		lowerViewer.book = upperViewer.book;
		lowerViewer.chapter = upperViewer.chapter;
		lowerViewer.verse = upperViewer.verse;

		VwrRelRedrawViewer(&lowerViewer, &upperViewer);
	} else {
		upperViewer.bound = fullBound;
		VwrRedrawViewer(&upperViewer);
	}

	if (doubleBuffered){
		WinCopyRectangle(bufferWin, drawWin, &fullBound, fullBound.topLeft.x, fullBound.topLeft.y, winPaint);
		WinSetDrawWindow(drawWin);
	}
}

static void moveViewer(WinDirectionType moveDir, Coord moveValue)
{
	WinHandle drawWin;
	
	if (doubleBuffered){
		drawWin = WinSetDrawWindow(bufferWin);
	}

	lastChapter = upperViewer.chapter;
	lastVerse = upperViewer.verse;

	if (dualView){
		if (upperView){
			VwrScrollViewer(&upperViewer, moveDir, moveValue);
			VwrRelScrollViewer(&lowerViewer, &upperViewer);
		} else {
			if (lowerViewer.enabled){
				VwrScrollViewer(&lowerViewer, moveDir, moveValue);
				VwrRelScrollViewer(&upperViewer, &lowerViewer);
			}
		}
	} else {
		VwrScrollViewer(&upperViewer, moveDir, moveValue);
	}
	

	if (doubleBuffered){
		WinCopyRectangle(bufferWin, drawWin, &fullBound, fullBound.topLeft.x, fullBound.topLeft.y, winPaint);
		WinSetDrawWindow(drawWin);
	}

	if (lastChapter != upperViewer.chapter || lastVerse != upperViewer.verse){
		drawTitleVerse();
	}
}

static void moveViewerLines(WinDirectionType moveDir, UInt16 moveLines)
{
	UInt16 quarter, half, i; 

	drawTitleVerse();

	quarter = lineSpace / 4;
	half = lineSpace - quarter * 2;

    // For HandEra Fix

	moveViewer(moveDir, quarter * 2);

	/*
	moveViewer(moveDir, quarter);
	moveViewer(moveDir, quarter);
	*/
	moveViewer(moveDir, half);

	if (moveLines >= 6){
		moveViewer(moveDir, lineSpace);

		for (i=0; i< (moveLines - 4) / 2; i++){
			moveViewer(moveDir, lineSpace * 2);
		}

		if (moveLines / 2 * 2 != moveLines){
			moveViewer(moveDir, lineSpace);
		}

		moveViewer(moveDir, lineSpace);
	} else {
		// For HandEra Fix
		
		moveViewer(moveDir, lineSpace * (moveLines - 2));

		/*
		for (i=0; i< moveLines - 2; i++){
			moveViewer(moveDir, lineSpace);
		}
		*/
	}

	moveViewer(moveDir, half);

	// For HandEra Fix
	moveViewer(moveDir, quarter * 2);
	
	/*
	moveViewer(moveDir, quarter);
	moveViewer(moveDir, quarter);
	*/

	redrawTitle = true;
}

static Boolean frmMain_OnPenUp(EventPtr event)
{
	if (penDowned){
		penDowned = false;
		
		redrawTitle = true;
		
		return true;
	} else {
		penDowned = false;
		return false;
	}
}

static Boolean frmMain_ggtUpperView_OnPenDown(EventPtr event)
{
	drawTitleVerse();

	upperView = true;
	penDowned = true;
	lastY = event->screenY;
	return true;
}

static Boolean frmMain_ggtLowerView_OnPenDown(EventPtr event)
{
	drawTitleVerse();

	upperView = false;
	penDowned = true;
	lastY = event->screenY;
	return true;
}

static Boolean frmMain_ggtView_OnPenDown(EventPtr event)
{
	drawTitleVerse();

	penDowned = true;
	lastY = event->screenY;
	return true;
}

static Boolean frmMain_ggtView_OnPenMove(EventPtr event)
{
	Coord newY;
	Coord moveValue;
	WinDirectionType moveDir;

	if (penDowned){
		newY = event->screenY;

		if (lastY < newY){
			moveDir = winDown;
			moveValue = newY - lastY;
		} else {
			moveDir = winUp;
			moveValue = lastY - newY;
		}

		if (moveValue > moveTH){
			moveViewer(moveDir, moveValue);
			lastY = newY;
		}

		return true;
	} else {
		return false;
	}
}

static Boolean frmMain_btnBookmark_OnSelect(EventPtr event)
{
	FrmPopupForm(frmBookmark);
	return true;
}

static Boolean frmMain_btnSearch_OnSelect(EventPtr event)
{
	gotoSearch();
	return true;
}

static Boolean frmMain_btnPref_OnSelect(EventPtr event)
{
	FrmPopupForm(frmPref);
	return true;
}

static Boolean frmMain_btnForward_OnSelect(EventPtr event)
{
	HtyForwardHistory();
	drawTitleVerse();

	redrawViewer();

	redrawTitle = true;
	return true;
}

static Boolean frmMain_btnBack_OnSelect(EventPtr event)
{
	HtyBackHistory();
	drawTitleVerse();

	redrawViewer();

	redrawTitle = true;
	return true;
}

static Boolean frmMain_btnGoto_OnSelect(EventPtr event)
{
	FrmPopupForm(frmGoto);
	return true;
}

static Boolean frmMain_OnUpdate(EventPtr event)
{
	if (event->data.frmUpdate.updateCode == viewUpdateCode){
		drawTitleVerse();

		redrawViewer();

		redrawTitle = true;
		return true;
	} else {
		// For HandEra Fix
		return true;

		//return false;
	}
}

static Boolean frmMain_OnKeyDown(EventPtr event)
{
	UInt32 keyState;
	UInt16 totalMove, i;

	if (dualView){
		totalMove = screenHeight / lineSpace / 2;
	} else {
		totalMove = screenHeight / lineSpace - 1;
	}

	keyState = KeyCurrentState();
	if (keyState & keyBitPageUp){
		moveViewerLines(winDown, totalMove);
	} else if (keyState & keyBitPageDown){
		moveViewerLines(winUp, totalMove);
	} else {
		return false;
	}

	return true;
}

static Boolean frmMain_OnNil(EventPtr event)
{
	if (redrawTitle){
		drawTitle(fmnTitle);
		redrawTitle = false;
	}
}

static Boolean frmMain_OnClose(EventPtr event)
{
	VwrCloseViewer(&upperViewer);
	VwrCloseViewer(&lowerViewer);
	
	if (doubleBuffered){
		WinDeleteWindow(bufferWin, false);
	}

	return false;
}

static Boolean frmMain_OnOpen(EventPtr event)
{
	UInt16 ggtViewIndex;
	UInt16 error;

	if (doubleBuffered){
		bufferWin = WinCreateOffscreenWindow(160, 160, screenFormat, &error);
	}
	
	form = FrmGetActiveForm();
	FrmDrawForm(form);
	
	fmnTitle = (char*) FrmGetTitle(form);

	ggtViewIndex = FrmGetObjectIndex(form, ggtView);
	FrmGetObjectBounds(form, ggtViewIndex, &fullBound);

	upperBound.topLeft = fullBound.topLeft;
	upperBound.extent.x = fullBound.extent.x;
	upperBound.extent.y = fullBound.extent.y / 2 - 2;

	lowerBound.topLeft.x = fullBound.topLeft.x;
	lowerBound.topLeft.y = fullBound.topLeft.y + upperBound.extent.y + 4;
	lowerBound.extent = upperBound.extent;
	
	if (dualView){
		upperViewer.bound = upperBound;
	} else {
		upperViewer.bound = fullBound;
	}
	lowerViewer.bound = lowerBound;
	
	VwrOpenViewer(&upperViewer);
	VwrOpenViewer(&lowerViewer);

	redrawTitle = false;

	redrawViewer();
	
	return true;
}

Boolean frmMain_HandleEvent(EventPtr event)
{
	Boolean handled = false;
	
	switch (event->eType)
	{
		case penUpEvent:
			handled = frmMain_OnPenUp(event);
			break;
		case penDownEvent:
			if (dualView) {
				if (RctPtInRectangle(event->screenX, event->screenY, &upperBound)) {
					handled = frmMain_ggtUpperView_OnPenDown(event);
				} else if (RctPtInRectangle(event->screenX, event->screenY, &lowerBound)) {
					handled = frmMain_ggtLowerView_OnPenDown(event);
				}
			} else {
				if (RctPtInRectangle(event->screenX, event->screenY, &fullBound)) {
					handled = frmMain_ggtView_OnPenDown(event);
				}
			}
			break;
		case penMoveEvent:
			if (RctPtInRectangle(event->screenX, event->screenY, &fullBound)) {
				handled = frmMain_ggtView_OnPenMove(event);
			}
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case btnBookmark:
					handled = frmMain_btnBookmark_OnSelect(event);
					break;
				case btnSearch:
					handled = frmMain_btnSearch_OnSelect(event);
					break;
				case btnPref:
					handled = frmMain_btnPref_OnSelect(event);
					break;
				case btnForward:
					handled = frmMain_btnForward_OnSelect(event);
					break;
				case btnBack:
					handled = frmMain_btnBack_OnSelect(event);
					break;
				case btnGoto:
					handled = frmMain_btnGoto_OnSelect(event);
					break;
			}
			break;
		case frmUpdateEvent:
			handled = frmMain_OnUpdate(event);
			break;
		case keyDownEvent:
			handled = frmMain_OnKeyDown(event);
			break;
		case nilEvent:
			handled = frmMain_OnNil(event);
			break;
		case frmCloseEvent:
			handled = frmMain_OnClose(event);
			break;
		case frmOpenEvent:
			handled = frmMain_OnOpen(event);
			break;
		default:
			break;
	}

	return handled;
}
