/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"   

static ReaderType fsrReader;
static char* fsrSearchText;
static Boolean fsrPenDowned;
static UInt16 fsrSelectedResult;

static Coord fsrResultTop;
static RectangleType fsrResultBound;
static ListContainPtr fsrBookList;
static UInt16 fsrProgressUsable; 

static char* fsrVerseBuffer;

static void drawResultBox()
{
	UInt16 i;

	fsrResultBound.topLeft.y = fsrResultTop;
	for (i=0; i<totalResultLine; i++){
		WinEraseRectangle(&fsrResultBound, 0);
		fsrResultBound.topLeft.y += fsrResultBound.extent.y;
	}
}

static void initResultBox()
{
	UInt16 objIndex;

	objIndex = FrmGetObjectIndex(modalForm, ggtResult);
	FrmGetObjectBounds(modalForm, objIndex, &fsrResultBound);

	fsrResultTop = fsrResultBound.topLeft.y;
	fsrResultBound.extent.y = fsrResultBound.extent.y / totalResultLine;
}

static void drawRawText(char* verseText, UInt16 line)
{
	char* verseNumText;

	verseNumText = InfGetSimpleFormattedVerse(searchVersion, result[line].book, 
		result[line].chapter, result[line].verse);

	WinDrawTruncChars(verseNumText, StrLen(verseNumText), 
		fsrResultBound.topLeft.x + 1,
		fsrResultBound.topLeft.y + 1, searchVerseWidth);

	WinDrawTruncChars(verseText, 
		StrLen(verseText), fsrResultBound.topLeft.x + searchVerseWidth + 1,
		fsrResultBound.topLeft.y + 1, 
		fsrResultBound.extent.x - searchVerseWidth - 1);

	MemPtrFree(verseNumText);
}

static void drawText(char* verseText, UInt16 line, Boolean inverse)
{
	IndexedColorType orgTextColor, orgForeColor, orgBackColor;
	
	fsrResultBound.topLeft.y = fsrResultTop + (fsrResultBound.extent.y) * line;

	if (os35){
    	if (inverse) {
    		orgForeColor = WinSetForeColor(
    			UIColorGetTableEntryIndex(
    			UIMenuSelectedFill));
    
    		WinDrawRectangle(&fsrResultBound, 3);
    
    		orgBackColor = WinSetBackColor(
    			UIColorGetTableEntryIndex(
    			UIMenuSelectedFill));
    
    		orgTextColor = WinSetTextColor(orgBackColor);
    	} else {
    		WinEraseRectangle(&fsrResultBound, 3);
    	}
    
    	drawRawText(verseText, line);
    
    	if (inverse) {
    		WinSetBackColor(orgBackColor);
    		WinSetTextColor(orgTextColor);
    		WinSetForeColor(orgForeColor);
    	}
	} else {
		if (inverse){
			WinInvertRectangle(&fsrResultBound, 3);
		} else {
			WinEraseRectangle(&fsrResultBound, 3);
			drawRawText(verseText, line);
		}
	} 
}

static void drawVerse(UInt16 i, Boolean inverse)
{
	RdrSeekVerse(&fsrReader, searchVersion, 
		result[i].book, result[i].chapter, result[i].verse);
			
	RdrGetCurrentVerseAsString(&fsrReader, fsrVerseBuffer);
	drawText(fsrVerseBuffer, i, inverse);
}

static void redrawVerse()
{
	UInt16 i;

	for (i=0; i<resultLine; i++){
		drawVerse(i, false);
	}
}

static Boolean getNextVerse()
{ 
	UInt16 bookIndex, newBook;
	VersePtr verse;

	if (searchProgress.book > searchTo.book){
		return false;
	} else if (searchProgress.book < searchFrom.book){
		return false;
	} else if (searchProgress.book == searchTo.book && 
		searchProgress.chapter == searchTo.chapter && 
		searchProgress.verse > searchTo.verse){

		return false;
	}

	result[resultLine].book = searchProgress.book;
	result[resultLine].chapter = fsrReader.chapter;
	result[resultLine].verse = fsrReader.verse;

	if (!RdrGetCurrentVerseAsString(&fsrReader, fsrVerseBuffer)){
		bookIndex = getListIndex(fsrBookList, searchProgress.book); 
		newBook = getListItem(fsrBookList, bookIndex + 1);
		
		if (newBook < searchProgress.book){
			return false;
		}
		
		searchProgress.book = newBook;
		searchProgress.chapter = 1;
		searchProgress.verse = 1;

		result[resultLine].book = searchProgress.book;
		result[resultLine].chapter = 1;
		result[resultLine].verse = 1;

		RdrSeekVerse(&fsrReader, searchVersion, 
			searchProgress.book, searchProgress.chapter, searchProgress.verse);

		RdrGetCurrentVerseAsString(&fsrReader, fsrVerseBuffer);
	}
	RdrMoveNextVerse(&fsrReader);
	
	searchProgress.book = fsrReader.book;
	searchProgress.chapter = fsrReader.chapter;
	searchProgress.verse = fsrReader.verse;
	
	return true;
}

static void search()
{
	UInt16 i, lastBook;
	char* found;
	char* bookName;
	Boolean labelUsable;
	
	lastBook = 0;
	labelUsable = searching;
	
	inSearch = true;
	
	setControlUsable(lblProgress, fsrProgressUsable);
	fsrProgressUsable++;
				
	if (fsrProgressUsable > 4){
		fsrProgressUsable = 0;
	}

	RdrSeekVerse(&fsrReader, searchVersion, 
		searchProgress.book, searchProgress.chapter, searchProgress.verse);

	if (resultLine == totalResultLine){
		resultLine = 0;

		drawResultBox();
	}

	for(i=0; i<searchRecordBreak && resultLine <totalResultLine; i++) {
		searching = true;

		if (getNextVerse()){
			if (searchProgress.book != lastBook){
				bookName = InfGetComplexBookName(searchVersion, searchProgress.book);
				FrmCopyLabel(modalForm, lblProgBookname, bookName);
				lastBook = searchProgress.book;
				MemPtrFree(bookName);
			}
			
			if (!labelUsable){
				setControlUsable(lblProgBookname, true);
				labelUsable = true;
			}

			found = StrStr(fsrVerseBuffer, fsrSearchText);

			if (found != NULL){
				if (resultLine == totalResultLine - 1){
					setControlUsable(lblProgress, false);
					setControlUsable(lblProgBookname, false);
				}
				
				drawText(fsrVerseBuffer, resultLine, false);
				resultLine++;
			}
		} else {
			searching = false;
			i = searchRecordBreak;
		}
	}

	if (i < searchRecordBreak || resultLine == totalResultLine){
		searching = false;
	}

	if (!searching){
		setControlUsable(lblProgress, false);
		setControlUsable(lblProgBookname, false);
	}
}

static Boolean frmSearchResult_cmdNext_OnSelect(EventPtr event)
{
	if (!searching){
		search();
	}
	return true;
}

static Boolean frmSearchResult_cmdNew_OnSelect(EventPtr event)
{
	inSearch = false;
	searching = false;
	FrmGotoForm(frmSearch);
	return true;
}

static Boolean frmSearchResult_cmdClose_OnSelect(EventPtr event)
{
	searching = false;
	FrmGotoForm(frmMain);
	return true;
}

static Boolean frmSearchResult_OnPenUp(EventPtr event)
{
	UInt16 i;

	for (i=0; i<resultLine; i++){
		if (i == fsrSelectedResult){

			drawVerse(i, false);

			fsrResultBound.topLeft.y = fsrResultTop + fsrResultBound.extent.y * i;
			if (RctPtInRectangle(event->screenX, event->screenY, &fsrResultBound)){
				HtyPushHistory();

				upperViewer.book = result[i].book;			
				upperViewer.chapter = result[i].chapter;			
				upperViewer.verse = result[i].verse;
				upperViewer.offset = 0;
				i = totalResultLine;

				FrmGotoForm(frmMain);
			}
		}
	}
	
	fsrPenDowned = false;
	return true;
}

static Boolean frmSearchResult_OnPenDown(EventPtr event)
{
	UInt16 i;

	for (i=0; i<resultLine; i++){

		fsrResultBound.topLeft.y = fsrResultTop + fsrResultBound.extent.y * i;

		if (RctPtInRectangle(event->screenX, event->screenY, &fsrResultBound)){
			fsrSelectedResult = i;

			drawVerse(i, true);

			fsrPenDowned = true;
			return true;
		}
	}
	fsrSelectedResult = totalResultLine;
	fsrPenDowned = false;

	return false;
}

static Boolean frmSearchResult_OnNil(EventPtr event)
{
	if (searching) {
		search();
	}
}

static frmSearchResult_OnClose(EventPtr event)
{
	searching = false;
	MemPtrFree(fsrVerseBuffer);
	MemHandleUnlock(searchTextHandle);
	RdrCloseReader(&fsrReader);
	ListPtrFree(fsrBookList);
	
	return false;
}

static Boolean frmSearchResult_OnOpen(EventPtr event)
{
	modalForm = FrmGetActiveForm();
	FrmDrawForm(modalForm);

	fsrVerseBuffer = MemPtrNew(searchBufferSize);
	fsrBookList = InfGetComplexBookList(searchVersion);

	initResultBox();
	fsrSearchText = MemHandleLock(searchTextHandle);
	
	RdrOpenReader(&fsrReader);
	fsrProgressUsable = 1;
	
	if (startSearch){
		searchProgress = searchFrom;
		resultLine = 0;
		drawResultBox();
		search();
	} else {
		drawResultBox();
		redrawVerse();
	}
	
	fsrSelectedResult = totalResultLine;
	fsrPenDowned = false;
	
	return true;
}

Boolean frmSearchResult_HandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)
	{
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case cmdNext:
					handled = frmSearchResult_cmdNext_OnSelect(event);
					break;
				case cmdNew:
					handled = frmSearchResult_cmdNew_OnSelect(event);
					break;
				case cmdClose:
					handled = frmSearchResult_cmdClose_OnSelect(event);
					break;
			}
			break;
		case penUpEvent:
			handled = frmSearchResult_OnPenUp(event);
			break;
		case penDownEvent:
			handled = frmSearchResult_OnPenDown(event);
			break;
		case nilEvent:
			handled = frmSearchResult_OnNil(event);
			break;
		case frmCloseEvent:
			handled = frmSearchResult_OnClose(event);
			break;
		case frmOpenEvent:
			handled = frmSearchResult_OnOpen(event);
			break;
	}

	return handled;
}
