/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"

static VersionInfoPtr infVersionInfoPtr;
static UInt16 infVersion;
static UInt16 infBook;
static UInt16 infChapter;
static UInt16 infVerse;

static void lockVersionPtr()
{
	char* versionName;
	char* versionInfo;
	char* sepChar;
	UInt16* wordIndex;
	UInt16* totalWordRec;
	UInt16* totalBooks;
	BookInfoPtr bookInfo;

	infVersionInfoPtr = DmxLockVersionRecord(infVersion, infVerInfoRecord);
}

static void unlockVersionPtr()
{
	DmxUnlockVersionRecord(infVersion, infVerInfoRecord);
}  

static BookInfoPtr lockBookPtr()
{
	BookInfoPtr bookInfo;
	BookInfoPtr resultBookInfo;

	UInt16 totalBooks;
	UInt16* bookPtr;
	UInt16 i;

	lockVersionPtr();

	totalBooks = infVersionInfoPtr->totalBooks;
	bookInfo = infVersionInfoPtr->bookInfo;
	
	resultBookInfo = NULL;
	
	for (i=0; i<totalBooks; i++){
		if (bookInfo[i].bookNum == infBook){
			resultBookInfo = &bookInfo[i];
			i = totalBooks;
		}
	}
	
	return resultBookInfo;
}

static void unlockBookPtr()
{
	unlockVersionPtr();
}

static char* getVersionInfo(Boolean name)
{
	UInt16 versionInfoLen;
	char* versionInfo;
	
	lockVersionPtr(infVersion);

	if (name){
		versionInfoLen = StrLen(infVersionInfoPtr->versionName);
		versionInfo	= MemPtrNew(versionInfoLen + 1);
		StrCopy(versionInfo, infVersionInfoPtr->versionName);
	} else {
		versionInfoLen = StrLen(infVersionInfoPtr->versionInfo);
		versionInfo	= MemPtrNew(versionInfoLen + 1);
		StrCopy(versionInfo, infVersionInfoPtr->versionInfo);
	}

	unlockVersionPtr();

	return versionInfo;
}

static UInt16 getWordRecord()
{
	UInt16 result;

	lockVersionPtr(infVersion);
	result = infVersionInfoPtr->wordIndex;
	unlockVersionPtr();

	return result;
}

static UInt16 getWordTotal()
{
	UInt16 result;

	lockVersionPtr(infVersion);
	result = infVersionInfoPtr->totalWordRec;
	unlockVersionPtr();

	return result;
}

static UInt16 getBookRecord()
{
	BookInfoPtr bookInfo;
	UInt16 bookRecord;
	UInt16* chapterInfoPtr;
	UInt16 totalChapter;
	UInt16 totalVerse;

	bookInfo = lockBookPtr();
	if (bookInfo != NULL){
		bookRecord = bookInfo->bookIndex;
	} else {
		bookRecord = 0;
	}
	unlockBookPtr();	

	return bookRecord;
}

static UInt16 getBookTotal()
{
	BookInfoPtr bookInfo;
	UInt16 bookRecord;
	UInt16* chapterInfoPtr;
	UInt16 totalChapter;
	UInt16 totalVerse;

	bookInfo = lockBookPtr();
	bookRecord = bookInfo->totalBookRec;
	unlockBookPtr();	

	return bookRecord;
}

static char* getBookName(Boolean complex)
{
	BookInfoPtr bookInfo;

	UInt16 bookNameLen;
	char* bookName;

	UInt16 i; 

	bookInfo = lockBookPtr(infVersion, infBook);

	if (bookInfo == NULL){
		bookName = NULL;
	} else {
		if (complex){
			bookNameLen = StrLen(bookInfo->complexName);
			bookName = MemPtrNew(bookNameLen + 1);
			StrCopy(bookName, bookInfo->complexName);
		} else {
			bookNameLen = StrLen(bookInfo->simpleName);
			bookName = MemPtrNew(bookNameLen + 1);
			StrCopy(bookName, bookInfo->simpleName);
		}
	}

	unlockBookPtr();
	
	return bookName;
}

static UInt16 getTotalBook()
{
	UInt16 totalBooks;

	lockVersionPtr(infVersion);
	totalBooks = infVersionInfoPtr->totalBooks;
	unlockVersionPtr();
	
	return totalBooks;
}

static ListContainPtr getBookList(Boolean complex)
{
	BookInfoPtr bookInfoPtr;
	ListContainPtr list;

	UInt16 bookNameLen;
	char* bookName;

	UInt16 totalBooks;
	UInt16 i;

	lockVersionPtr(infVersion);
	totalBooks = infVersionInfoPtr->totalBooks;
	bookInfoPtr = infVersionInfoPtr->bookInfo;
	
	list = ListPtrNew(totalBooks);

	for (i=0; i<totalBooks; i++){
		if (complex){
			bookNameLen = StrLen(bookInfoPtr[i].complexName);
			bookName = MemPtrNew(bookNameLen + 1);
			StrCopy(bookName, bookInfoPtr[i].complexName);
		} else {
			bookNameLen = StrLen(bookInfoPtr[i].simpleName);
			bookName = MemPtrNew(bookNameLen + 1);
			StrCopy(bookName, bookInfoPtr[i].simpleName);
		}

		list->nameList[i] = bookName;
		list->numList[i] = bookInfoPtr[i].bookNum;
	};

	unlockVersionPtr();

	return list;
}

char* getFormattedVerse(Boolean complex)
{
	char* bookText;
	char* currentText;
	char* resultText;

	bookText = getBookName(complex);
	
	if (bookText == NULL){
		resultText = NULL;
	} else {
		currentText = MemPtrNew(64);
		StrPrintF(currentText, "%s %d%s%d", bookText, infChapter, ":"
			, infVerse);

		resultText = MemPtrNew(StrLen(currentText) + 1);
		StrCopy(resultText, currentText);

		MemPtrFree(currentText);
		MemPtrFree(bookText);
	}
	
	return resultText;
}

// Public Functions

void InfOpenInfo()
{
}

void InfCloseInfo()
{
}

UInt16 InfGetTotalVersion()
{
	return DmxGetTotalVersions();
}

char InfGetSepChar(UInt16 version)
{
	UInt16 result;

	lockVersionPtr(version);
	result = infVersionInfoPtr->sepChar;
	unlockVersionPtr();

	return result;
}

Boolean InfGetRightAligned(UInt16 version)
{
	UInt8 result;

	infVersion = version; 

	lockVersionPtr(version);
	result = infVersionInfoPtr->versionAttrib;
	unlockVersionPtr();

	return result & infRightAligned;
}

Boolean InfGetCopyProtected(UInt16 version)
{
	UInt8 result;

	infVersion = version; 

	lockVersionPtr(version);
	result = infVersionInfoPtr->versionAttrib;
	unlockVersionPtr();

	return result & infCopyProtected;
}

Boolean InfGetByteShifted(UInt16 version)
{
	UInt8 result;

	infVersion = version; 

	lockVersionPtr(version);
	result = infVersionInfoPtr->versionAttrib;
	unlockVersionPtr();

	return !(result & infByteNotShifted);
}

char* InfGetVersionInfo(UInt16 version)
{
	infVersion = version; 
	return getVersionInfo(false);
}

char* InfGetVersionName(UInt16 version)
{
	infVersion = version; 
	return getVersionInfo(true);
}

ListContainPtr InfGetVersionList()
{
	ListContainPtr list;
	UInt16 totalVersion;
	UInt16 i;

	totalVersion = DmxGetTotalVersions();
	list = ListPtrNew(totalVersion);

	for (i=0; i<totalVersion; i++){
		infVersion = i + 1;
		list->nameList[i] = getVersionInfo(true);
		list->numList[i] = i + 1;
	};

	sortListPtr(list, true, true);
	
	return list;
}

UInt16 InfGetTotalBook(UInt16 version)
{
	infVersion = version;

	return getTotalBook(); 
}

char* InfGetSimpleBookName(UInt16 version, UInt16 book)
{
	infVersion = version; 
	infBook = book; 

	return getBookName(false);
}

char* InfGetComplexBookName(UInt16 version, UInt16 book)
{
	infVersion = version; 
	infBook = book; 

	return getBookName(true);
}

ListContainPtr InfGetSimpleBookList(UInt16 version)
{
	infVersion = version; 
	return getBookList(false);
}

ListContainPtr InfGetComplexBookList(UInt16 version)
{
	infVersion = version; 
	return getBookList(true);
}

UInt16 InfGetBookRecordIndex(UInt16 version, UInt16 book)
{
	infVersion = version;
	infBook = book; 

	return getBookRecord(); 
}

UInt16 InfGetBookTotalRecord(UInt16 version, UInt16 book)
{
	infVersion = version;
	infBook = book; 

	return getBookTotal(); 
}

UInt16 InfGetWordRecordIndex(UInt16 version)
{
	infVersion = version;

	return getWordRecord(); 
}

UInt16 InfGetWordTotalRecord(UInt16 version)
{
	infVersion = version;

	return getWordTotal(); 
}

UInt16 InfGetTotalChapter(UInt16 version, UInt16 book)
{
	BookInfoPtr bookInfo;
	UInt16 bookRecord;
	UInt16* chapterInfoPtr;
	UInt16 totalChapter;

	infVersion = version;
	infBook = book; 
	
	bookRecord = getBookRecord();
	chapterInfoPtr = DmxLockVersionRecord(infVersion, bookRecord);
	totalChapter = chapterInfoPtr[0];
	DmxUnlockVersionRecord(infVersion, bookRecord);

	return totalChapter;
}

UInt16 InfGetTotalVerse(UInt16 version, UInt16 book, UInt16 chapter)
{
	UInt16 bookRecord;
	UInt16* chapterInfoPtr;
	UInt16 totalChapter;
	UInt16 totalVerse;

	infVersion = version;
	infBook = book; 

	bookRecord = getBookRecord();

	chapterInfoPtr = DmxLockVersionRecord(infVersion, bookRecord);
	totalChapter = chapterInfoPtr[0];
	if (chapter <= totalChapter){
		totalVerse = chapterInfoPtr[chapter]; 
		totalVerse -= (chapter == 1) ? 0:chapterInfoPtr[chapter - 1];
	} else {
		totalVerse = 0;
	}
	DmxUnlockVersionRecord(infVersion, bookRecord);

	return totalVerse;
}

char* InfGetSimpleFormattedVerse(UInt16 version, UInt16 book, UInt16 chapter, UInt16 verse)
{
	infVersion = version;
	infBook = book;
	infChapter = chapter; 
	infVerse = verse; 

	return getFormattedVerse(false);
}

char* InfGetComplexFormattedVerse(UInt16 version, UInt16 book, UInt16 chapter, UInt16 verse)
{
	infVersion = version;
	infBook = book;
	infChapter = chapter; 
	infVerse = verse; 

	return getFormattedVerse(true);
}

