/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"

// As PalmOS is single threaded, make it global and reserve names
// for cleaner coding. No parameters passing between internal functions.

static ViewerPtr vwrViewer;
static WinDirectionType vwrMoveDir;
static Coord vwrMoveValue;
static RectangleType vwrRedrawBound;
static RectangleType vwrRedrawTestBound;
static UInt16 vwrVerse;
static Boolean vwrVBoldEnable;

static void clearVerse()
{
	VersePtr verseEntry;
	VersePtr lastVerseEntry;

	lastVerseEntry = NULL;
	verseEntry = vwrViewer->begin;

	while (verseEntry != NULL){
		lastVerseEntry = verseEntry;
		verseEntry = verseEntry->next;
		VersePtrFree(lastVerseEntry);
	}

	vwrViewer->begin = NULL;
	vwrViewer->totalLines = 0;
}

static void fillVerse()
{
	VersePtr verseEntry;
	VersePtr lastVerseEntry;
	ReaderPtr reader;
	
	reader = &vwrViewer->reader;

	lastVerseEntry = NULL;
	verseEntry = vwrViewer->begin;

	if (vwrViewer->totalLines < (middleBuffer + assumeLen + vwrViewer->begin->lines) || lastVerseEntry == NULL){
        RdrBookmark(reader);
        
        while (verseEntry != NULL){
            RdrMoveNextVerse(reader);
            lastVerseEntry = verseEntry;
            verseEntry = verseEntry->next;
        }

		while (lastVerseEntry == NULL || vwrViewer->totalLines < (middleBuffer + vwrViewer->begin->lines + lastVerseEntry->lines)){
			verseEntry = RdrGetCurrentVerse(reader);

			if (verseEntry != NULL){
				vwrViewer->totalLines += verseEntry->lines;

				if (lastVerseEntry != NULL){
					lastVerseEntry->next = verseEntry;
					verseEntry->prev = lastVerseEntry;
				} else {
					vwrViewer->begin = verseEntry;
				}

				RdrMoveNextVerse(reader);
				lastVerseEntry = verseEntry;
			} else {
				break;
			}
		}
		
		RdrReturn(reader);
	} else if (vwrViewer->totalLines > (middleBuffer + assumeLen + vwrViewer->begin->lines)){
        while (verseEntry != NULL){
            lastVerseEntry = verseEntry;
            verseEntry = verseEntry->next;
        }

		while (vwrViewer->totalLines > (middleBuffer + vwrViewer->begin->lines + lastVerseEntry->lines)){
			verseEntry = lastVerseEntry->prev;
			verseEntry->next = NULL;
			vwrViewer->totalLines -= lastVerseEntry->lines;
			VersePtrFree(lastVerseEntry);
			lastVerseEntry = verseEntry;
		}
	}
}

static Boolean movePrevVerse()
{
	VersePtr prev;
	VersePtr next;
	VersePtr begin;

	RdrMovePrevVerse(&vwrViewer->reader);
	prev = RdrGetCurrentVerse(&vwrViewer->reader);

	if (prev != NULL){
		vwrViewer->chapter = prev->chapter;
		vwrViewer->verse = prev->verse;

		begin = vwrViewer->begin;

		prev->next = begin;
		begin->prev = prev;

		vwrViewer->begin = prev;
		vwrViewer->totalLines += prev->lines;

		if (vwrViewer->totalLines > (upperBuffer + assumeLen + vwrViewer->begin->lines)){
			fillVerse();
		}
		return true;
	} else {
		RdrMoveNextVerse(&vwrViewer->reader);
		return false;
	}
}

static Boolean moveNextVerse()
{
	VersePtr prev;
	VersePtr next;
	VersePtr begin;

	RdrMoveNextVerse(&vwrViewer->reader);
	next = vwrViewer->begin->next;

	if (next != NULL){
		vwrViewer->chapter = next->chapter;
		vwrViewer->verse = next->verse;
		
		begin = vwrViewer->begin;

		next->prev = NULL;
		vwrViewer->begin = next;
		vwrViewer->totalLines -= begin->lines;

		VersePtrFree(begin);

		if (vwrViewer->totalLines < (lowerBuffer + assumeLen + vwrViewer->begin->lines)){
			fillVerse();
		}
		return true;
	} else {
		RdrMovePrevVerse(&vwrViewer->reader);
		return false;
	}
}

static void setViewerByViewer(ViewerPtr relViewer)
{
	UInt16 versePixels, relVersePixels, i;
	Int16 moveDir;
	Boolean cont;
	
	vwrMoveValue = -vwrViewer->offset;

	cont = true;
	versePixels = vwrViewer->begin->lines * lineSpace;
	while (vwrViewer->chapter > relViewer->chapter && cont){
		cont = movePrevVerse();
		versePixels = vwrViewer->begin->lines * lineSpace;
		vwrMoveValue -= versePixels;
	}
	
	cont = true;
	while (vwrViewer->chapter < relViewer->chapter && cont){
		versePixels = vwrViewer->begin->lines * lineSpace;
		vwrMoveValue += versePixels;
		cont = moveNextVerse();
	}

	cont = true;
	while (vwrViewer->verse > relViewer->verse 
		&& vwrViewer->chapter == relViewer->chapter && cont){
		cont = movePrevVerse();
		versePixels = vwrViewer->begin->lines * lineSpace;
		vwrMoveValue -= versePixels;
	}

	cont = true;
	while (vwrViewer->verse < relViewer->verse
		&& vwrViewer->chapter == relViewer->chapter && cont){
		versePixels = vwrViewer->begin->lines * lineSpace;
		vwrMoveValue += versePixels;
		cont = moveNextVerse();
	}

	versePixels = vwrViewer->begin->lines * lineSpace;
	if (vwrViewer->chapter == relViewer->chapter && vwrViewer->verse == relViewer->verse) {
		relVersePixels = relViewer->begin->lines * lineSpace;
		
		if (relVersePixels != 0){
			vwrViewer->offset = (UInt32) relViewer->offset * (UInt32) versePixels / (UInt32) relVersePixels;
		} else {
			vwrViewer->offset = 0;
		}
	} else {
		vwrViewer->offset = 0;
	}
	vwrMoveValue += vwrViewer->offset;

	if (vwrMoveValue < 0){
		vwrMoveDir = winDown;
		vwrMoveValue = -vwrMoveValue;
	} else {
		vwrMoveDir = winUp;
	}
}

static void setViewerByMove()
{
	Int16 versePixels;
	Int16 scroll;
	Int16 newOffSet;

	scroll = 0;
	newOffSet = vwrViewer->offset;
	
	if (vwrMoveDir == winDown){
		newOffSet -= vwrMoveValue;

		while (newOffSet < 0 && movePrevVerse()){
			versePixels = vwrViewer->begin->lines * lineSpace;
			newOffSet += versePixels;
		}

		if (newOffSet < 0){
			scroll = vwrMoveValue + newOffSet;
			newOffSet = 0;
		} else {
			scroll = vwrMoveValue;
		}
	} else {
		newOffSet += vwrMoveValue;
		versePixels = vwrViewer->begin->lines * lineSpace;

		while (newOffSet > versePixels && moveNextVerse()){
			newOffSet -= versePixels;
			versePixels = vwrViewer->begin->lines * lineSpace;
		}

		if (newOffSet > versePixels + 1){
			scroll = newOffSet - vwrMoveValue;
			newOffSet = versePixels;
		} else {
			scroll = vwrMoveValue;
		}
	}

	vwrViewer->offset = newOffSet;
	vwrMoveValue = scroll;
}

static void drawTextLine(char* textLine, Coord offset)
{
	PointType vwrTextLineTL;

	vwrTextLineTL = vwrViewer->bound.topLeft;
	vwrTextLineTL.y += offset;

    RenDrawLine(textLine, vwrVerse, vwrTextLineTL.y, vwrVBoldEnable, 
	    vwrViewer->rightAligned);
}

static void drawViewer()
{
	VersePtr verseEntry;
	UInt16 i, j;
	UInt16 baseX, baseY, offset;
	Boolean drawn, cont;

	baseX = vwrViewer->bound.topLeft.x;
	baseY = vwrViewer->bound.topLeft.y;
	
	vwrVBoldEnable = vwrViewer->reader.decoder.sepChar == 0;

	WinSetClip(&(vwrViewer->bound));
	
	WinEraseRectangle(&vwrRedrawBound, 0);

	vwrRedrawTestBound.topLeft.y -= lineSpace;
	vwrRedrawTestBound.extent.y += lineSpace;

	verseEntry = vwrViewer->begin;
	
	drawn = false;
	cont = true;
	j=0;

	while(cont && verseEntry != NULL){
		vwrVerse = verseEntry->verse;
		for (i=0; i< verseEntry->lines; i++){
			offset = j * lineSpace - vwrViewer->offset;

        	if (RctPtInRectangle(baseX, baseY + offset, &vwrRedrawTestBound)){
        		RenDrawLine(verseEntry->textLines[i], vwrVerse, baseY + offset, 
					vwrVBoldEnable, vwrViewer->rightAligned);
				drawn = true;
        	} else {
				if (drawn){
					cont = false;
				}
			}

			j++;
		}
		verseEntry = verseEntry->next;
	}

	WinResetClip();
}

static void scrollViewer()
{
	if (vwrMoveValue > 0) {
    	WinSetClip(&(vwrViewer->bound));
    	WinScrollRectangle(&(vwrViewer->bound), vwrMoveDir, vwrMoveValue, &vwrRedrawBound);

    	vwrRedrawTestBound = vwrRedrawBound;

    	drawViewer();
    	WinResetClip();
	}
}

static void drawMessage(UInt16 messageID)
{
	MemHandle messageHandle;
	char* messagePtr;
	char* messageNPtr;

	messageHandle = DmGetResource('tSTR', messageID);
	messagePtr = MemHandleLock(messageHandle);
	messageNPtr = MemPtrNew(StrLen(messagePtr) + 2);
	messageNPtr[0] = 'd';
	StrCopy(&messageNPtr[1], messagePtr);

	drawTextLine(messageNPtr, lineSpace);
	
	MemHandleUnlock(messageHandle);
	MemPtrFree(messageNPtr);
}

static void refillViewer()
{
	vwrRedrawBound = vwrViewer->bound;
	vwrRedrawTestBound = vwrViewer->bound;

	clearVerse();

	if (vwrViewer->version !=0 && !InfGetCopyProtected(vwrViewer->version)){
		if (RdrSeekVerse(&(vwrViewer->reader), vwrViewer->version, 
			vwrViewer->book, vwrViewer->chapter, vwrViewer->verse)){
			
			vwrViewer->rightAligned = InfGetRightAligned(vwrViewer->version);
			fillVerse();
			vwrViewer->enabled = true;
		} else {
			drawMessage(strBookNotFound);
			vwrViewer->enabled = false;
		}
	} else {
		WinEraseRectangle(&vwrRedrawBound, 0);

		drawMessage(strCannotReadBible);
		vwrViewer->enabled = false;
	}
}

// public functions

void VwrOpenViewer(ViewerPtr viewer)
{
	UInt16 error;
	char sepChar;
	
	viewer->totalLines = 0;
	viewer->begin = NULL;
	RdrOpenReader(&(viewer->reader));
};

void VwrCloseViewer(ViewerPtr viewer)
{
	RdrCloseReader(&(viewer->reader));
	clearVerse();
}

void VwrScrollViewer(ViewerPtr viewer, WinDirectionType moveDir, Coord moveValue)
{
	if (viewer->enabled){
		vwrViewer = viewer;
		vwrMoveDir = moveDir;
		vwrMoveValue = moveValue;

		setViewerByMove();
		scrollViewer();
		fillVerse();
	}
}

void VwrRelScrollViewer(ViewerPtr viewer, ViewerPtr relViewer)
{
	if (viewer->enabled){
		vwrViewer = viewer;

		setViewerByViewer(relViewer);
		scrollViewer();
		fillVerse();
	}
}

void VwrRedrawViewer(ViewerPtr viewer)
{
	vwrViewer = viewer;

	refillViewer();
	if (vwrViewer->enabled){
		drawViewer();
	}
}

void VwrRelRedrawViewer(ViewerPtr viewer, ViewerPtr relViewer)
{
	vwrViewer = viewer;

	refillViewer();
	if (vwrViewer->enabled){
		setViewerByViewer(relViewer);
		drawViewer();
	}
}
